"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.initializeGitRepository = exports.installPackageDependencies = exports.copyTemplateFiles = exports.generatePackageJSONfile = exports.bootstrapProject = exports.validateProjectPath = exports.validateTemplate = exports.validateProjectName = void 0;
const ajv_1 = __importDefault(require("ajv"));
const child_process_1 = require("child_process");
const fs_extra_1 = __importDefault(require("fs-extra"));
const os_1 = __importDefault(require("os"));
const path_1 = __importDefault(require("path"));
const util_1 = __importDefault(require("util"));
const validate_npm_package_name_1 = __importDefault(require("validate-npm-package-name"));
const constants_1 = require("./constants");
const utils_1 = require("./utils");
const execAsync = util_1.default.promisify(child_process_1.exec);
const validateProjectName = async (name) => {
    const results = (0, validate_npm_package_name_1.default)(name);
    if (!results.validForNewPackages) {
        const { errors, warnings } = results;
        let messages = [];
        if (errors) {
            messages = [...messages, ...errors];
        }
        if (warnings) {
            messages = [...messages, ...warnings];
        }
        throw new Error((0, utils_1.capitalize)(messages.join(', ')));
    }
};
exports.validateProjectName = validateProjectName;
const validateTemplate = async (template) => {
    const ajv = new ajv_1.default();
    const validate = ajv.compile(constants_1.MANIFEST_SCHEMA);
    const templateFolders = fs_extra_1.default
        .readdirSync(constants_1.TEMPLATES_PATH)
        .filter((file) => {
        const filePath = `${constants_1.TEMPLATES_PATH}/${file}`;
        return fs_extra_1.default.statSync(filePath).isDirectory() && fs_extra_1.default.existsSync(`${filePath}/manifest.json`);
    })
        .filter((templateFolder) => {
        const manifest = JSON.parse(fs_extra_1.default.readFileSync(`${constants_1.TEMPLATES_PATH}/${templateFolder}/manifest.json`, 'utf8'));
        return validate(manifest);
    });
    const avaliableTemplates = templateFolders.map((templateFolder) => {
        const manifestPath = path_1.default.join(constants_1.TEMPLATES_PATH, templateFolder, 'manifest.json');
        const manifest = JSON.parse(fs_extra_1.default.readFileSync(manifestPath, 'utf8'));
        return manifest.name;
    });
    if (!avaliableTemplates.includes(template)) {
        const templates = avaliableTemplates
            .map((t, index) => (index === avaliableTemplates.length - 1 ? `or "${t}"` : `"${t}"`))
            .join(', ');
        throw new Error(`The template "${template}" is not supported yet. Please use ${templates}`);
    }
};
exports.validateTemplate = validateTemplate;
const validateProjectPath = async (appPath) => {
    if (fs_extra_1.default.existsSync(appPath)) {
        throw new Error(`The path: "${appPath}" is not empty`);
    }
};
exports.validateProjectPath = validateProjectPath;
const bootstrapProject = async (appPath) => {
    fs_extra_1.default.ensureDirSync(appPath);
};
exports.bootstrapProject = bootstrapProject;
const generatePackageJSONfile = async (name, template, appPath) => {
    const packageInfo = {
        name,
        version: '0.1.0',
        private: true
    };
    const manifestPath = path_1.default.join(constants_1.TEMPLATES_PATH, template, 'manifest.json');
    const { scripts } = JSON.parse(fs_extra_1.default.readFileSync(manifestPath, 'utf8'));
    const packageJson = { ...packageInfo, scripts };
    fs_extra_1.default.writeFileSync(path_1.default.join(appPath, 'package.json'), JSON.stringify(packageJson, null, 2) + os_1.default.EOL);
};
exports.generatePackageJSONfile = generatePackageJSONfile;
const copyTemplateFiles = async (template, appPath) => {
    const manifestPath = path_1.default.join(constants_1.TEMPLATES_PATH, template, 'manifest.json');
    const { files, basePath } = JSON.parse(fs_extra_1.default.readFileSync(manifestPath, 'utf8'));
    const templateBasePath = path_1.default.join(constants_1.TEMPLATES_PATH, template, basePath);
    files.forEach((file) => {
        const filePath = path_1.default.join(templateBasePath, file);
        const fileExists = fs_extra_1.default.pathExistsSync(filePath);
        if (!fileExists) {
            throw `The file: "${path_1.default.basename(file)}" was not found in the "${template}" template files  `;
        }
        // Rename gitignore after the fact to prevent npm from renaming it to .npmignore
        // See: https://github.com/npm/npm/issues/1862
        const fileName = file === 'gitignore' ? '.gitignore' : file;
        const destPath = path_1.default.join(appPath, fileName);
        fs_extra_1.default.ensureDirSync(path_1.default.dirname(destPath));
        fs_extra_1.default.copyFileSync(filePath, destPath);
    });
};
exports.copyTemplateFiles = copyTemplateFiles;
const installPackageDependencies = async (template, appPath) => {
    const manifestPath = path_1.default.join(constants_1.TEMPLATES_PATH, template, 'manifest.json');
    const { dependencies } = JSON.parse(fs_extra_1.default.readFileSync(manifestPath, 'utf8'));
    process.chdir(appPath);
    if (dependencies.dev.length > 0) {
        await execAsync(`npm i -D ${dependencies.dev.join(' ')}`);
    }
    if (dependencies.prod.length > 0) {
        await execAsync(`npm i ${dependencies.prod.join(' ')}`);
    }
};
exports.installPackageDependencies = installPackageDependencies;
const initializeGitRepository = async (appPath) => {
    process.chdir(appPath);
    await execAsync(`git init`);
};
exports.initializeGitRepository = initializeGitRepository;
